==> bar-1.03.tar.gz: bar-1.03/bar.c <==
#include "config.h"

#ifndef STDC_HEADERS
#error "Argh!  This system has non-standard header files!"
#endif

#ifdef STAT_MACROS_BROKEN
#error "Argh!  This system has broken sys/stat.h macros!"
#endif

#include "fd.h"
#include "io.h"
#include "display.h"

#include "help.h"

#include <stdio.h>
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif
#ifdef HAVE_SYS_STAT_H
#include <sys/stat.h>
#endif
#ifdef HAVE_FCNTL_H
#include <fcntl.h>
#endif
#include <time.h>
#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif
#include <ctype.h>

void version(FILE *out)
{
	fprintf(out, "%s version %s\n", PACKAGE, VERSION);
}

void usage(FILE *out)
{
	unsigned int c = 0;

	for (c = 0; help_text[c] != 0; c++) {
		fprintf(out, "%s\n", help_text[c]);
	}
}

int isOpt(char *s1, char *s2)
{
	if (strcmp(s1, s2) == 0) return(1);
	return(0);
}

int same(int k, unsigned long long i, char *s)
{
	char str[1024] = { 0 };
	char fmt[12] = { 0 };
	int length = 0;
	int precision = 0;
	int precision_flag = 0;
	int c = 0;
	unsigned long long div = 1;
	unsigned long long n = 0;
	float f = 0.0;
	char unit = 0;

	n = i;
	if (!isdigit((int)(s[strlen(s)-1]))) {
		unit = s[strlen(s)-1];
	}
	switch (toupper(unit)) {
		case 'K':
			n /= 1;
			div = k;
			break;
		case 'M':
			n /= k;
			div = k;
			break;
		case 'G':
			n /= k;
			n /= k;
			div = k;
			break;
		case 'T':
			n /= k;
			n /= k;
			n /= k;
			div = k;
			break;
	}
	f = (float)((float)n / div);

	for (c = 0; c < strlen(s); c++) {
		if (isdigit((int)(s[c])) || s[c] == '.') {
			length++;
			if (precision_flag) {
				precision++;
			}
		}
		if (s[c] == '.') {
			precision_flag = 1;
		}
	}

	if (precision_flag) {
		sprintf(fmt, "%%%d.%df", length, precision);
		sprintf(str, fmt, f);
	}
	else {
		sprintf(fmt, "%%%d.%dd", length, length);
		sprintf(str, fmt, (int)f);
	}

	if ((unit != 0) && (strlen(str) != strlen(s)-1)) return(0);
	if ((unit == 0) && (strlen(str) != strlen(s))) return(0);
	if (unit != 0) {
		str[strlen(s)-1] = unit;
		if (strcmp(s, str) != 0) return(0);
	}
	return(1);
}

int main(int argc, char *argv[])
{
	int c = 0;
	float f = 0.0;
	
	if (ioInit() != 0) return(1);
	if (displayInit() != 0) return(1);

	for (c = 1; c < argc; c++) {
		if (isOpt(argv[c], "-k") || isOpt(argv[c], "--kilo")) {
			if (c+1 == argc) {
				fprintf(stderr, "*** ERROR: Missing option after %s\n", argv[c]);
				return(1);
			}
			if (strcmp(argv[c+1], "1000") == 0) {
				d.k = 1000;
			}
			else if (strcmp(argv[c+1], "1024") == 0) {
				d.k = 1024;
			}
			else {
				fprintf(stderr, "*** ERROR: Invalid option for %s: %s\n", 
					argv[c], argv[c+1]);
				return(1);
			}
			c++;
		}
	}

	for (c = 1; c < argc; c++) {
		if (isOpt(argv[c], "-h") || isOpt(argv[c], "--help")) {
			version(stdout);
			fprintf(stdout, "\n");
			usage(stdout);
			return(0);
		}
		else if (isOpt(argv[c], "-v") || isOpt(argv[c], "--version")) {
			version(stdout);
			return(0);
		}
		else if (isOpt(argv[c], "-if") || isOpt(argv[c], "--in-file")) {
			if (c+1 == argc) {
				fprintf(stderr, "*** ERROR: Missing filename after %s\n", argv[c]);
				return(1);
			}
			if (!isOpt(argv[c+1], "-")) {
				io.in = open(argv[c+1], O_RDONLY);
				if (io.in < 0) {
					fprintf(stderr, "*** ERROR: Cannot open file for reading: %s\n",
						argv[c+1]);
					return(1);
				}
				if (fdIsFile(io.in) && fdFileSize(io.in, &(io.total_size)) == 0) {
					io.total_size_known = 1;
				}
			}
			c++;
		}
		else if (isOpt(argv[c], "-of") || isOpt(argv[c], "--out-file")) {
			if (c+1 == argc) {
				fprintf(stderr, "*** ERROR: Missing filename after %s\n", argv[c]);
				return(1);
			}
			if (!isOpt(argv[c+1], "-")) {
				io.out = open(argv[c+1], O_WRONLY);
				if (io.out < 0) {
					fprintf(stderr, "*** ERROR: Cannot open file for writing: %s\n",
						argv[c+1]);
					return(1);
				}
			}
			c++;
		}
		else if (isOpt(argv[c], "-s") || isOpt(argv[c], "--size")) {
			if (c+1 == argc) {
				fprintf(stderr, "*** ERROR: Missing size after %s\n", argv[c]);
				return(1);
			}
			sscanf(argv[c+1], "%g", &f);
			switch (toupper(argv[c+1][strlen(argv[c+1])-1])) {
				case 'K':
					io.total_size = f 
						* (unsigned long long)d.k;
					break;
				case 'M':
					io.total_size = f 
						* (unsigned long long)d.k
						* (unsigned long long)d.k;
					break;
				case 'G':
					io.total_size = f 
						* (unsigned long long)d.k
						* (unsigned long long)d.k
						* (unsigned long long)d.k;
					break;
				case 'T':
					io.total_size = f 
						* (unsigned long long)d.k
						* (unsigned long long)d.k
						* (unsigned long long)d.k
						* (unsigned long long)d.k;
					break;
				default:
					io.total_size = (unsigned long long)f;
					break;
			}
			if (!same(d.k, io.total_size, argv[c+1])) {
				fprintf(stderr, "*** ERROR: Type mismatch or number too large: %s\n",
					argv[c+1]);
				fprintf(stderr, "           Ignoring given size\n");
				io.total_size = 0;
			}
			else {
				io.total_size_known = 1;
			}
			c++;
		}
		else if (isOpt(argv[c], "-bs") || isOpt(argv[c], "--buffer-size")) {
			if (c+1 == argc) {
				fprintf(stderr, "*** ERROR: Missing size after %s\n", argv[c]);
				return(1);
			}
			sscanf(argv[c+1], "%g", &f);
			switch (toupper(argv[c+1][strlen(argv[c+1])-1])) {
				case 'K':
					io.buffer_size = f
						* (size_t)d.k;
					break;
				case 'M':
					io.buffer_size = f
						* (size_t)d.k
						* (size_t)d.k;
					break;
				case 'G':
					io.buffer_size = f
						* (size_t)d.k
						* (size_t)d.k
						* (size_t)d.k;
					break;
				case 'T':
					io.buffer_size = f
						* (size_t)d.k
						* (size_t)d.k
						* (size_t)d.k
						* (size_t)d.k;
					break;
				default:
					io.buffer_size = (size_t)f;
					break;
			}
			if (!same(d.k, io.buffer_size, argv[c+1])) {
				fprintf(stderr, "*** ERROR: Type mismatch or number too large: %s\n",
					argv[c+1]);
				fprintf(stderr, "           Ignoring given buffer size, using a 1k buffer\n");
				io.buffer_size = d.k;
			}
			c++;
		}
		else if (isOpt(argv[c], "-i") || isOpt(argv[c], "--interval")) {
			if (c+1 == argc) {
				fprintf(stderr, "*** ERROR: Missing integer after %s\n", argv[c]);
				return(1);
			}
			sscanf(argv[c+1], "%d", &d.display_interval);
			if ((d.display_interval < 1) || (d.display_interval > 60*60*24)) {
				fprintf(stderr, "*** ERROR: Invalid display interval: %s\n",
					argv[c+1]);
				return(1);
			}
			c++;
		}
		else if (isOpt(argv[c], "-t") || isOpt(argv[c], "--timeout")) {
			if (c+1 == argc) {
				fprintf(stderr, "*** ERROR: Missing microseconds after %s\n",
					argv[c]);
				return(1);
			}
			sscanf(argv[c+1], "%lu", &io.timeout);
			if (!same(1000, io.timeout, argv[c+1]) || (io.timeout > 999999)) {
				fprintf(stderr, "*** ERROR: Type mismatch or number too large: %s\n",
					argv[c+1]);
				fprintf(stderr, "           Ignoring given timeout, using 250000 microseconds\n");
				io.timeout = 250000;
			}
			c++;
		}
		else if (isOpt(argv[c], "-k") || isOpt(argv[c], "--kilo")) {
			/* eat it */
			c++;
		}
		else {
			fprintf(stderr, "*** ERROR: Unknown command line option: %s\n",
				argv[c]);
			usage(stderr);
			return(1);
		}
	}

	if (ioBegin() != 0) return(1);
	if (displayBegin() != 0) return(1);

	while (!ioIsDone()) {
		ioCheck();
		ioRead();
		ioWrite();
		displayUpdate();
	}

	displayEnd();
	ioEnd();
	return(0);
}

==> bar-1.04.tar.gz: bar-1.04/bar.c <==
#include "config.h"

#ifndef STDC_HEADERS
#error "Argh!  This system has non-standard header files!"
#endif

#ifdef STAT_MACROS_BROKEN
#error "Argh!  This system has broken sys/stat.h macros!"
#endif

#include "types.h"
#include "fd.h"
#include "io.h"
#include "display.h"

#include "help.h"

#include <stdio.h>
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif
#ifdef HAVE_SYS_STAT_H
#include <sys/stat.h>
#endif
#ifdef HAVE_FCNTL_H
#include <fcntl.h>
#endif
#include <time.h>
#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif
#include <ctype.h>

void version(FILE *out)
{
	fprintf(out, "%s version %s\n", PACKAGE, VERSION);
}

void usage(FILE *out)
{
	unsigned int c = 0;

	for (c = 0; help_text[c] != 0; c++) {
		fprintf(out, "%s\n", help_text[c]);
	}
}

int isOpt(char *s1, char *s2)
{
	if (strcmp(s1, s2) == 0) return(1);
	return(0);
}

int same(int k, uint64 i, char *s)
{
	char str[1024] = { 0 };
	char fmt[12] = { 0 };
	int length = 0;
	int precision = 0;
	int precision_flag = 0;
	int c = 0;
	uint64 div = 1;
	uint64 n = 0;
	float f = 0.0;
	char unit = 0;

	n = i;
	if (!isdigit((int)(s[strlen(s)-1]))) {
		unit = s[strlen(s)-1];
	}
	switch (toupper(unit)) {
		case 'K':
			n /= 1;
			div = k;
			break;
		case 'M':
			n /= k;
			div = k;
			break;
		case 'G':
			n /= k;
			n /= k;
			div = k;
			break;
		case 'T':
			n /= k;
			n /= k;
			n /= k;
			div = k;
			break;
		case 'P':
			n /= k;
			n /= k;
			n /= k;
			n /= k;
			div = k;
			break;
		case 'E':
			n /= k;
			n /= k;
			n /= k;
			n /= k;
			n /= k;
			div = k;
			break;
	}
	f = (float)((float)n / div);

	for (c = 0; c < strlen(s); c++) {
		if (isdigit((int)(s[c])) || s[c] == '.') {
			length++;
			if (precision_flag) {
				precision++;
			}
		}
		if (s[c] == '.') {
			precision_flag = 1;
		}
	}

	if (precision_flag) {
		sprintf(fmt, "%%%d.%df", length, precision);
		sprintf(str, fmt, f);
	}
	else {
		sprintf(fmt, "%%%d.%dd", length, length);
		sprintf(str, fmt, (int)f);
	}

	if ((unit != 0) && (strlen(str) != strlen(s)-1)) return(0);
	if ((unit == 0) && (strlen(str) != strlen(s))) return(0);
	if (unit != 0) {
		str[strlen(s)-1] = unit;
		if (strcmp(s, str) != 0) return(0);
	}
	return(1);
}

int main(int argc, char *argv[])
{
	int c = 0;
	float f = 0.0;
	
	if (ioInit() != 0) return(1);
	if (displayInit() != 0) return(1);

	for (c = 1; c < argc; c++) {
		if (isOpt(argv[c], "-k") || isOpt(argv[c], "--kilo")) {
			if (c+1 == argc) {
				fprintf(stderr, "*** ERROR: Missing option after %s\n", argv[c]);
				return(1);
			}
			if (strcmp(argv[c+1], "1000") == 0) {
				d.k = 1000;
			}
			else if (strcmp(argv[c+1], "1024") == 0) {
				d.k = 1024;
			}
			else {
				fprintf(stderr, "*** ERROR: Invalid option for %s: %s\n", 
					argv[c], argv[c+1]);
				return(1);
			}
			c++;
		}
	}

	for (c = 1; c < argc; c++) {
		if (isOpt(argv[c], "-h") || isOpt(argv[c], "--help")) {
			version(stdout);
			fprintf(stdout, "\n");
			usage(stdout);
			return(0);
		}
		else if (isOpt(argv[c], "-v") || isOpt(argv[c], "--version")) {
			version(stdout);
			return(0);
		}
		else if (isOpt(argv[c], "-if") || isOpt(argv[c], "--in-file")) {
			if (c+1 == argc) {
				fprintf(stderr, "*** ERROR: Missing filename after %s\n", argv[c]);
				return(1);
			}
			if (!isOpt(argv[c+1], "-")) {
				io.in = open(argv[c+1], O_RDONLY);
				if (io.in < 0) {
					fprintf(stderr, "*** ERROR: Cannot open file for reading: %s\n",
						argv[c+1]);
					return(1);
				}
				if (fdIsFile(io.in) && fdFileSize(io.in, &(io.total_size)) == 0) {
					io.total_size_known = 1;
				}
			}
			c++;
		}
		else if (isOpt(argv[c], "-of") || isOpt(argv[c], "--out-file")) {
			if (c+1 == argc) {
				fprintf(stderr, "*** ERROR: Missing filename after %s\n", argv[c]);
				return(1);
			}
			if (!isOpt(argv[c+1], "-")) {
				io.out = open(argv[c+1], O_WRONLY);
				if (io.out < 0) {
					fprintf(stderr, "*** ERROR: Cannot open file for writing: %s\n",
						argv[c+1]);
					return(1);
				}
			}
			c++;
		}
		else if (isOpt(argv[c], "-s") || isOpt(argv[c], "--size")) {
			if (c+1 == argc) {
				fprintf(stderr, "*** ERROR: Missing size after %s\n", argv[c]);
				return(1);
			}
			sscanf(argv[c+1], "%g", &f);
			switch (toupper(argv[c+1][strlen(argv[c+1])-1])) {
				case 'K':
					io.total_size = f 
						* (uint64)d.k;
					break;
				case 'M':
					io.total_size = f 
						* (uint64)d.k
						* (uint64)d.k;
					break;
				case 'G':
					io.total_size = f 
						* (uint64)d.k
						* (uint64)d.k
						* (uint64)d.k;
					break;
				case 'T':
					io.total_size = f 
						* (uint64)d.k
						* (uint64)d.k
						* (uint64)d.k
						* (uint64)d.k;
					break;
				case 'P':
					io.total_size = f 
						* (uint64)d.k
						* (uint64)d.k
						* (uint64)d.k
						* (uint64)d.k
						* (uint64)d.k;
					break;
				case 'E':
					io.total_size = f 
						* (uint64)d.k
						* (uint64)d.k
						* (uint64)d.k
						* (uint64)d.k
						* (uint64)d.k
						* (uint64)d.k;
					break;
				default:
					io.total_size = (uint64)f;
					break;
			}
			if (!same(d.k, io.total_size, argv[c+1])) {
				fprintf(stderr, "*** ERROR: Type mismatch or number too large: %s\n",
					argv[c+1]);
				fprintf(stderr, "           Ignoring given size\n");
				io.total_size = 0;
			}
			else {
				io.total_size_known = 1;
			}
			c++;
		}
		else if (isOpt(argv[c], "-bs") || isOpt(argv[c], "--buffer-size")) {
			if (c+1 == argc) {
				fprintf(stderr, "*** ERROR: Missing size after %s\n", argv[c]);
				return(1);
			}
			sscanf(argv[c+1], "%g", &f);
			switch (toupper(argv[c+1][strlen(argv[c+1])-1])) {
				case 'K':
					io.buffer_size = f
						* (size_t)d.k;
					break;
				case 'M':
					io.buffer_size = f
						* (size_t)d.k
						* (size_t)d.k;
					break;
				case 'G':
					io.buffer_size = f
						* (size_t)d.k
						* (size_t)d.k
						* (size_t)d.k;
					break;
				case 'T':
					io.buffer_size = f
						* (size_t)d.k
						* (size_t)d.k
						* (size_t)d.k
						* (size_t)d.k;
					break;
				case 'P':
					io.buffer_size = f
						* (size_t)d.k
						* (size_t)d.k
						* (size_t)d.k
						* (size_t)d.k
						* (size_t)d.k;
					break;
				case 'E':
					io.buffer_size = f
						* (size_t)d.k
						* (size_t)d.k
						* (size_t)d.k
						* (size_t)d.k
						* (size_t)d.k
						* (size_t)d.k;
					break;
				default:
					io.buffer_size = (size_t)f;
					break;
			}
			if (!same(d.k, io.buffer_size, argv[c+1])) {
				fprintf(stderr, "*** ERROR: Type mismatch or number too large: %s\n",
					argv[c+1]);
				fprintf(stderr, "           Ignoring given buffer size, using a 1k buffer\n");
				io.buffer_size = d.k;
			}
			c++;
		}
		else if (isOpt(argv[c], "-i") || isOpt(argv[c], "--interval")) {
			if (c+1 == argc) {
				fprintf(stderr, "*** ERROR: Missing integer after %s\n", argv[c]);
				return(1);
			}
			sscanf(argv[c+1], "%d", &d.display_interval);
			if ((d.display_interval < 1) || (d.display_interval > 60*60*24)) {
				fprintf(stderr, "*** ERROR: Invalid display interval: %s\n",
					argv[c+1]);
				return(1);
			}
			c++;
		}
		else if (isOpt(argv[c], "-t") || isOpt(argv[c], "--timeout")) {
			if (c+1 == argc) {
				fprintf(stderr, "*** ERROR: Missing microseconds after %s\n",
					argv[c]);
				return(1);
			}
			sscanf(argv[c+1], "%lu", &io.timeout);
			if (!same(1000, io.timeout, argv[c+1]) || (io.timeout > 999999)) {
				fprintf(stderr, "*** ERROR: Type mismatch or number too large: %s\n",
					argv[c+1]);
				fprintf(stderr, "           Ignoring given timeout, using 250000 microseconds\n");
				io.timeout = 250000;
			}
			c++;
		}
		else if (isOpt(argv[c], "-k") || isOpt(argv[c], "--kilo")) {
			/* eat it */
			c++;
		}
		else if (isOpt(argv[c], "-sw") || isOpt(argv[c], "--screen-width")) {
			if (c+1 == argc) {
				fprintf(stderr, "*** ERROR: Missing integer after %s\n", argv[c]);
				return(1);
			}
			sscanf(argv[c+1], "%d", &d.screen_width);
			/*
			 * The upper-end of this range should be modified to something sane:
			 */
			if ((d.screen_width < 1) || (d.screen_width > 500)) {
				fprintf(stderr, "*** ERROR: Invalid screen width: %s\n", argv[c+1]);
				return(1);
			}
			c++;
		}
		else if (isOpt(argv[c], "-ntw") || isOpt(argv[c], "--no-twiddle")) {
			d.display_twiddle = 0;
		}
		else if (isOpt(argv[c], "-nc") || isOpt(argv[c], "--no-count")) {
			d.display_datacount = 0;
		}
		else if (isOpt(argv[c], "-nth") || isOpt(argv[c], "--no-throughput")) {
			d.display_throughput = 0;
		}
		else if (isOpt(argv[c], "-nt") || isOpt(argv[c], "--no-time")) {
			d.display_time = 0;
		}
		else if (isOpt(argv[c], "-np") || isOpt(argv[c], "--no-percent")) {
			d.display_percent = 0;
		}
		else if (isOpt(argv[c], "-nb") || isOpt(argv[c], "--no-bar")) {
			d.display_bar = 0;
		}
		else if (isOpt(argv[c], "-dtw") || isOpt(argv[c], "--display-twiddle")) {
			d.display_twiddle = 1;
		}
		else if (isOpt(argv[c], "-dc") || isOpt(argv[c], "--display-count")) {
			d.display_datacount = 1;
		}
		else if (isOpt(argv[c], "-dth") || isOpt(argv[c], "--display-throughput")) {
			d.display_throughput = 1;
		}
		else if (isOpt(argv[c], "-dt") || isOpt(argv[c], "--display-time")) {
			d.display_time = 1;
		}
		else if (isOpt(argv[c], "-dp") || isOpt(argv[c], "--display-percent")) {
			d.display_percent = 1;
		}
		else if (isOpt(argv[c], "-db") || isOpt(argv[c], "--display-bar")) {
			d.display_bar = 1;
		}
		else if (isOpt(argv[c], "-dn") || isOpt(argv[c], "--display-none")) {
			d.display_twiddle = 0;
			d.display_datacount = 0;
			d.display_throughput = 0;
			d.display_time = 0;
			d.display_percent = 0;
			d.display_bar = 0;
		}
		else if (isOpt(argv[c], "-da") || isOpt(argv[c], "--display-all")) {
			d.display_twiddle = 1;
			d.display_datacount = 1;
			d.display_throughput = 1;
			d.display_time = 1;
			d.display_percent = 1;
			d.display_bar = 1;
		}
		else {
			fprintf(stderr, "*** ERROR: Unknown command line option: %s\n",
				argv[c]);
			usage(stderr);
			return(1);
		}
	}

	if (ioBegin() != 0) return(1);
	if (displayBegin() != 0) return(1);

	while (!ioIsDone()) {
		ioCheck();
		ioRead();
		ioWrite();
		displayUpdate();
	}

	displayEnd();
	ioEnd();
	return(0);
}
